#ifndef GIAC_VECTOR_H
#define GIAC_VECTOR_H
#ifndef GIAC_VECTOR
#include <vector>
#else
// Simple vector implementation for giac
// Incomplete (allocation is handled by new/delete)
// compatible with alias_ref_vecteur and without all VISUALC++ checks

namespace std {

  inline void swapptr(void * a,void * b){ register void * c=a; a=b; b=c; }

  template<class _Tp> class reverse_pointer {
    _Tp * ptr;
  public:
  reverse_pointer(): ptr(0) {}
  reverse_pointer(_Tp *p): ptr(p) {}
  reverse_pointer(const reverse_pointer & p): ptr(p.ptr) {}
    reverse_pointer operator +(int n){ return ptr-n; }
    reverse_pointer operator -(int n){ return ptr+n; }
    reverse_pointer & operator ++(){ --ptr; return *this; }
    reverse_pointer & operator --(){ ++ptr; return *this; }
    _Tp & operator *() { return *ptr; }
    const _Tp operator *() const { return *ptr; }
    bool operator ==(const reverse_pointer & other) const { return ptr==other.ptr; }
    bool operator !=(const reverse_pointer & other) const { return ptr!=other.ptr; }
    int operator -(const reverse_pointer & other) const { return other.ptr-ptr; }
  };

  template<typename _Tp> class vector{
    // private members
    _Tp * _begin,*_end,*_endalloc;
    // private allocation methods
    void _realloc(unsigned n){
      if (_endalloc-_begin>=int(n))
	return;
      unsigned old=_end-_begin;
      _Tp * _newbegin = new _Tp[n];
      for (_Tp * ptr=_begin;ptr!=_end;++ptr,++_newbegin){
	*_newbegin = *ptr;
      }
      _newbegin -= old;
      if (_begin)
	delete [] _begin;
      _begin=_newbegin;
      _end=_begin+old;
      _endalloc=_begin+n;
    }
    void _alloc(unsigned n){
      _end=_begin = new _Tp[n];
      _endalloc=_begin+n;
    }
    void _alloc_fill(const _Tp * b,const _Tp * e){
      unsigned n=e-b;
      _alloc(n); 
      for (_Tp * ptr=_begin;ptr!=_endalloc;++ptr,++b){
	*ptr = *b;
      }
      _end = _begin+ n;
    }
  public:
    typedef _Tp value_type;
    typedef _Tp * pointer;
    typedef const _Tp * const_iterator;
    typedef pointer iterator;
    typedef reverse_pointer<_Tp> reverse_iterator;
    typedef reverse_pointer<const _Tp> const_reverse_iterator;
    
  vector():_begin(0),_end(0),_endalloc(0) {}
    ~vector() { if (_begin) delete [] _begin; }
    vector(unsigned n,const _Tp & value=_Tp()){
      _alloc(n); 
      for (;_end!=_endalloc;++_end){
	*_end =value;
      }
    }
    vector(const const_iterator & b,const const_iterator & e){
      _alloc_fill(b,e);
    }
    vector(const vector<_Tp> & w){
      _alloc_fill(w._begin,w._end);
    }
    vector<_Tp> & operator = (const vector<_Tp> & w){
      if (this==&w)
	return *this;
      _end=_begin;
      unsigned n=w._end-w._begin;
      _realloc(n);
      for (_Tp * ptr=w._begin;ptr!=w._end;++_end,++ptr){
	*_end = *ptr;
      }
      return *this;
    }
    iterator begin(){ return _begin; }
    iterator end(){ return _end;}
    const_iterator begin() const { return _begin; }
    const_iterator end() const { return _end;}
    reverse_iterator rbegin(){ return _end-1; }
    reverse_iterator rend(){ return _begin-1;}
    const_reverse_iterator rbegin() const { return _end-1; }
    const_reverse_iterator rend() const { return _begin-1;}
    unsigned size() const { return _end-_begin; }
    unsigned capacity() const { return _endalloc-_begin;}
    _Tp & front() { return *_begin; }
    _Tp & back() { return *(_end-1); }
    _Tp & operator [](unsigned i) { return *(_begin+i); }
    const _Tp & front() const { return *_begin; }
    const _Tp & back() const { return *(_end-1); }
    const _Tp & operator [](unsigned i) const { return *(_begin+i); }
    void push_back(const _Tp & p){ 
      if (_endalloc==_end){
	unsigned n = _end-_begin;
	_realloc(n?2*n:2);
      } 
      *_end=p; 
      ++_end; 
    }
    _Tp pop_back(){ --_end; return *_end; }
    void clear(){ _end=_begin;}
    bool empty() const { return _end==_begin; }
    void reserve(unsigned n){ if (_endalloc-_begin<int(n)) _realloc(n); }
    void resize(unsigned n,const _Tp &value=_Tp()){ 
      if (_end-_begin>=(int)n) _end=_begin+n;
      else {
	_realloc(n);
	for (unsigned i=_end-_endalloc;i<n;++i){
	  *_end=value;
	  ++_end;
	}
      }
    }
    void erase(_Tp * b,_Tp * e){
      unsigned decal=e-b;
      if (!decal)
	return;
      for (_Tp * ptr=e;ptr!=_end;++ptr){
	*(ptr-decal)=*ptr;
      }
      _end -= decal;
    }
    void erase(_Tp * b){
      erase(b,b+1);
    }
    _Tp * insert(_Tp * b, const _Tp& x ){
      if (_endalloc==_end){
	unsigned pos=b-_begin;
	unsigned n = _end-_begin;
	_realloc(n?2*n:2);
	b=_begin+pos;
      }
      ++_end;
      for (_Tp * ptr=_end-1;ptr!=b;--ptr){
	*ptr=*(ptr-1);
      }
      *b=x;
      return b;
    }
    void insert(_Tp * b, unsigned k,const _Tp& x ){
      if (_endalloc<_end+k){
	unsigned pos=b-_begin;
	unsigned n = _end-_begin;
	_realloc(n>k?2*n:n+k);
	b=_begin+pos;
      }
      _end += k ;
      for (_Tp * ptr=_end-k;ptr!=b;){
	--ptr;
	*(ptr+k)=*ptr;
      }
      for (unsigned j=0;j<k;++b,++j){
	*b=x;
      }
    }
    void swap(vector<_Tp> & w){
      swapptr((void *)_begin,(void *)w._begin);
      swapptr((void *)_end,(void *)w._end);
      swapptr((void *)_endalloc,(void *)w._endalloc);
    }
    void assign(unsigned n,const _Tp & value=_Tp()){
      _realloc(n);
      _end = _begin +n;
      for (_Tp * ptr=_begin;ptr!=_end;++ptr){
	*ptr=value;
      }
    }
    void assign(const _Tp * b,const _Tp * e){
      unsigned n=e-b;
      _realloc(n);
      _end = _begin +n;
	for (_Tp * ptr=_begin;b!=e;++b,++ptr){
	*ptr=*b;
      }
    }
    unsigned max_size() const {
      return 1 << 30;
    }
    _Tp & at(unsigned n){
      if (n>_end-_begin)
	return _Tp(); // should be defined somewhere else
      return *(_begin+n);
    }
    const _Tp at(unsigned n) const {
      if (n>_end-_begin)
	return _Tp(); // should be defined somewhere else
      return *(_begin+n);
    }
  };

  template<typename _Tp>
    inline bool operator==(const vector<_Tp>& __x, const vector<_Tp>& __y){ 
    if (__x.size() != __y.size())
      return false;
    for (const _Tp * xptr=__x.begin(), * yptr=__y.begin();xptr!=__x.end();++yptr,++xptr){
      if (*xptr!=*yptr)
	return false;
    }
    return true;
  }

  template<typename _Tp>
    inline bool operator < (const vector<_Tp>& __x, const vector<_Tp>& __y){ 
    if (__x.size() != __y.size())
      return __x.size()<__y.size();
    for (const _Tp * xptr=__x.begin(), * yptr=__y.begin();xptr!=__x.end();++yptr,++xptr){
      if (*xptr!=*yptr)
	return *xptr<*yptr;
    }
    return false;
  }

  template<typename _Tp>
    inline bool operator!=(const _Tp & __x, const _Tp & __y){
    return !(__x==__y);
  }

  template<typename _Tp>
    inline bool operator > (const _Tp & __x, const _Tp & __y){
    return __y<__x;
  }

  template<typename _Tp>
    inline bool operator >= (const _Tp & __x, const _Tp & __y){
    return !(__x<__y);
  }

  template<typename _Tp>
    inline bool operator <= (const _Tp & __x, const _Tp & __y){
    return !(__y<__x);
  }

}

#endif

#endif // GIAC_VECTOR_H
